@@ -229,6 +229,46 @@ class File:
         print(file)
 
 
+class Image:
+    @classmethod
+    def create(cls, args):
+        resp = openai.Image.create(
+            prompt=args.prompt,
+            size=args.size,
+            n=args.num_images,
+            response_format=args.response_format,
+        )
+        print(resp)
+
+    @classmethod
+    def create_variation(cls, args):
+        with open(args.image, "rb") as file_reader:
+            buffer_reader = BufferReader(file_reader.read(), desc="Upload progress")
+        resp = openai.Image.create_variation(
+            image=buffer_reader,
+            size=args.size,
+            n=args.num_images,
+            response_format=args.response_format,
+        )
+        print(resp)
+
+    @classmethod
+    def create_edit(cls, args):
+        with open(args.image, "rb") as file_reader:
+            image_reader = BufferReader(file_reader.read(), desc="Upload progress")
+        with open(args.mask, "rb") as file_reader:
+            mask_reader = BufferReader(file_reader.read(), desc="Upload progress")
+        resp = openai.Image.create_edit(
+            image=image_reader,
+            mask=mask_reader,
+            prompt=args.prompt,
+            size=args.size,
+            n=args.num_images,
+            response_format=args.response_format,
+        )
+        print(resp)
+
+
 class Search:
     @classmethod
     def prepare_data(cls, args, purpose):
@@ -240,7 +280,7 @@ class Search:
         optional_fields = ["metadata"]
 
         if purpose == "classifications":
-            required_fields = ["text", "labels"]
+            required_fields = ["text", "label"]
         else:
             required_fields = ["text"]
 
@@ -265,11 +305,11 @@ class Search:
         )
 
     @classmethod
-    def create_alpha(cls, args):
-        resp = openai.Search.create_alpha(
-            query=[args.query],
-            max_documents=args.max_documents,
-            file_id=args.file,
+    def create(cls, args):
+        resp = openai.Search.create(
+            query=args.query,
+            documents=args.documents,
+            model=args.model,
         )
         print(resp)
 
@@ -319,7 +359,9 @@ class FineTune:
                 sys.stdout.write(
                     "Found potentially duplicated files with name '{name}', purpose 'fine-tune' and size {size} bytes\n".format(
                         name=os.path.basename(matching_files[0]["filename"]),
-                        size=matching_files[0]["bytes"],
+                        size=matching_files[0]["bytes"]
+                        if "bytes" in matching_files[0]
+                        else matching_files[0]["size"],
                     )
                 )
                 sys.stdout.write("\n".join(file_ids))
@@ -393,11 +435,11 @@ class FineTune:
 
         for hparam in (
             "model",
+            "suffix",
             "n_epochs",
             "batch_size",
             "learning_rate_multiplier",
             "prompt_loss_weight",
-            "use_packing",
             "compute_classification_metrics",
             "classification_n_classes",
             "classification_positive_class",
@@ -536,6 +578,21 @@ class FineTune:
         )
 
 
+class WandbLogger:
+    @classmethod
+    def sync(cls, args):
+        import openai.wandb_logger
+
+        resp = openai.wandb_logger.WandbLogger.sync(
+            id=args.id,
+            n_fine_tunes=args.n_fine_tunes,
+            project=args.project,
+            entity=args.entity,
+            force=args.force,
+        )
+        print(resp)
+
+
 def tools_register(parser):
     subparsers = parser.add_subparsers(
         title="Tools", help="Convenience client side tools"
@@ -595,7 +652,7 @@ def tools_register(parser):
         action="store_true",
         help="Auto accepts all suggestions, without asking for user input. To be used within scripts.",
     )
-    sub.set_defaults(func=partial(Search.prepare_data, purpose="classification"))
+    sub.set_defaults(func=partial(Search.prepare_data, purpose="classifications"))
 
     sub = subparsers.add_parser("answers.prepare_data")
     sub.add_argument(
@@ -813,20 +870,14 @@ Mutually exclusive with `top_p`.""",
     sub.set_defaults(func=File.list)
 
     # Search
-    sub = subparsers.add_parser("search.create_alpha")
+    sub = subparsers.add_parser("search.create")
 
     sub.add_argument(
-        "-f",
-        "--file",
-        required=True,
-        help="ID for previously uploaded file that contains the documents you want to search",
-    )
-    sub.add_argument(
-        "-m",
-        "--max_documents",
-        help="The maximum number of documents to return",
-        type=int,
-        default=200,
+        "-d",
+        "--documents",
+        help="Documents to search over",
+        type=str,
+        nargs="+",
     )
     sub.add_argument(
         "-q",
@@ -834,7 +885,12 @@ Mutually exclusive with `top_p`.""",
         required=True,
         help="Search query",
     )
-    sub.set_defaults(func=Search.create_alpha)
+    sub.add_argument(
+        "-m",
+        "--model",
+        help="The model to search with",
+    )
+    sub.set_defaults(func=Search.create)
 
     # Finetune
     sub = subparsers.add_parser("fine_tunes.list")
@@ -867,6 +923,17 @@ Mutually exclusive with `top_p`.""",
         "--model",
         help="The model to start fine-tuning from",
     )
+    sub.add_argument(
+        "--suffix",
+        help="If set, this argument can be used to customize the generated fine-tuned model name."
+        "All punctuation and whitespace in `suffix` will be replaced with a "
+        "single dash, and the string will be lower cased. The max "
+        "length of `suffix` is 40 chars. "
+        "The generated name will match the form `{base_model}:ft-{org-title}:{suffix}-{timestamp}`. "
+        'For example, `openai api fine_tunes.create -t test.jsonl -m ada --suffix "custom model name" '
+        "could generate a model with the name "
+        "ada:ft-your-org:custom-model-name-2022-02-15-04-21-04",
+    )
     sub.add_argument(
         "--no_follow",
         action="store_true",
@@ -891,23 +958,6 @@ Mutually exclusive with `top_p`.""",
         "learning rate is determined by the original learning rate used for "
         "pretraining multiplied by this value.",
     )
-    sub.add_argument(
-        "--use_packing",
-        action="store_true",
-        dest="use_packing",
-        help="On classification tasks, we recommend not setting this flag. "
-        "On all other tasks, we recommend setting it. "
-        "When set, we pack as many prompt-completion pairs as possible into each "
-        "training example. This greatly increases the speed of a fine-tuning job, "
-        "often without negatively affecting model performance.",
-    )
-    sub.add_argument(
-        "--no_packing",
-        action="store_false",
-        dest="use_packing",
-        help="Disables the packing flag (see --use_packing for description).",
-    )
-    sub.set_defaults(use_packing=None)
     sub.add_argument(
         "--prompt_loss_weight",
         type=float,
@@ -972,3 +1022,88 @@ Mutually exclusive with `top_p`.""",
     sub = subparsers.add_parser("fine_tunes.cancel")
     sub.add_argument("-i", "--id", required=True, help="The id of the fine-tune job")
     sub.set_defaults(func=FineTune.cancel)
+
+    # Image
+    sub = subparsers.add_parser("image.create")
+    sub.add_argument("-p", "--prompt", type=str, required=True)
+    sub.add_argument("-n", "--num-images", type=int, default=1)
+    sub.add_argument(
+        "-s", "--size", type=str, default="1024x1024", help="Size of the output image"
+    )
+    sub.add_argument("--response-format", type=str, default="url")
+    sub.set_defaults(func=Image.create)
+
+    sub = subparsers.add_parser("image.create_edit")
+    sub.add_argument("-p", "--prompt", type=str, required=True)
+    sub.add_argument("-n", "--num-images", type=int, default=1)
+    sub.add_argument(
+        "-I",
+        "--image",
+        type=str,
+        required=True,
+        help="Image to modify. Should be a local path and a PNG encoded image.",
+    )
+    sub.add_argument(
+        "-s", "--size", type=str, default="1024x1024", help="Size of the output image"
+    )
+    sub.add_argument("--response-format", type=str, default="url")
+    sub.add_argument(
+        "-M",
+        "--mask",
+        type=str,
+        required=True,
+        help="Path to a mask image. It should be the same size as the image you're editing and a RGBA PNG image. The Alpha channel acts as the mask.",
+    )
+    sub.set_defaults(func=Image.create_edit)
+
+    sub = subparsers.add_parser("image.create_variation")
+    sub.add_argument("-n", "--num-images", type=int, default=1)
+    sub.add_argument(
+        "-I",
+        "--image",
+        type=str,
+        required=True,
+        help="Image to modify. Should be a local path and a PNG encoded image.",
+    )
+    sub.add_argument(
+        "-s", "--size", type=str, default="1024x1024", help="Size of the output image"
+    )
+    sub.add_argument("--response-format", type=str, default="url")
+    sub.set_defaults(func=Image.create_variation)
+
+
+def wandb_register(parser):
+    subparsers = parser.add_subparsers(
+        title="wandb", help="Logging with Weights & Biases"
+    )
+
+    def help(args):
+        parser.print_help()
+
+    parser.set_defaults(func=help)
+
+    sub = subparsers.add_parser("sync")
+    sub.add_argument("-i", "--id", help="The id of the fine-tune job (optional)")
+    sub.add_argument(
+        "-n",
+        "--n_fine_tunes",
+        type=int,
+        default=None,
+        help="Number of most recent fine-tunes to log when an id is not provided. By default, every fine-tune is synced.",
+    )
+    sub.add_argument(
+        "--project",
+        default="GPT-3",
+        help="""Name of the project where you're sending runs. By default, it is "GPT-3".""",
+    )
+    sub.add_argument(
+        "--entity",
+        help="Username or team name where you're sending runs. By default, your default entity is used, which is usually your username.",
+    )
+    sub.add_argument(
+        "--force",
+        action="store_true",
+        help="Forces logging and overwrite existing wandb run of the same fine-tune.",
+    )
+    sub.set_defaults(force=False)
+    sub.set_defaults(func=WandbLogger.sync)
